//+------------------------------------------------------------------+
//|                                                        VLDMI.mq4 |
//|                                       Copyright(C) 2015, PeakyFx |
//|                                      http://peakyfx.blogspot.jp/ |
//+------------------------------------------------------------------+
#property copyright   "Copyright(C) 2015, PeakyFx"
#property link        "http://peakyfx.blogspot.jp/"
#property version     "1.00"
#property description "医学的FX   http://peakyfx.blogspot.jp/"
#property strict

//---- indicator settings
#property indicator_separate_window
#property indicator_buffers 1
#property indicator_plots   1
#property indicator_color1  Pink
#property indicator_maximum 100
#property indicator_minimum   0

#property indicator_levelcolor Gray
#property indicator_levelstyle STYLE_DOT
#property indicator_levelwidth 1
#property indicator_level1 30
#property indicator_level2 70

//---- input parameters
extern int BasePeriod = 14;
extern int AvgPeriod  = 10;
extern int StdvPeriod = 5;
extern ENUM_APPLIED_PRICE AppliedPrice = PRICE_CLOSE;

//---- indicator buffers
double VlDmi[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
{
//--- indicator buffers mapping
	SetIndexBuffer(0, VlDmi);
	SetIndexStyle(0, DRAW_LINE);
	SetIndexLabel(0, StringFormat("VLDMI(%d)", BasePeriod));
	SetIndexDrawBegin(0, BasePeriod + AvgPeriod + StdvPeriod/2 + 1);
	
//---- indicator short name
	IndicatorShortName(StringFormat("VLDMI(%d,%d,%d)", BasePeriod, AvgPeriod, StdvPeriod));
	
//---
	return(INIT_SUCCEEDED);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
	int limit = rates_total - MathMax(prev_calculated, BasePeriod + AvgPeriod + StdvPeriod/2 + 1);
	
	for (int i = limit; i >= 0; i--)
	{
		VlDmi[i] = iVLDMI(NULL, 0, BasePeriod, AvgPeriod, StdvPeriod, AppliedPrice, i);
	}
	
//--- return value of prev_calculated for next call
	return(rates_total);
}

//+------------------------------------------------------------------+
double iVI(string symbol, int timeframe, int period, int stdv_period, ENUM_APPLIED_PRICE applied_price, int shift) export
{
	double std_dev = iStdDev(symbol, timeframe, stdv_period, 0, MODE_SMA, applied_price, shift);
	double stdv_sum = std_dev;
		
	for (int i = 1; i < period; i++)
	{
		stdv_sum += iStdDev(symbol, timeframe, stdv_period, 0, MODE_SMA, applied_price, shift + i);
	}
	
	if (std_dev==0 || stdv_sum==0) return 1;
	return std_dev / stdv_sum * period;
}

double iVLDMI(string symbol, int timeframe, int base_period, int avg_period, int stdv_period,
				ENUM_APPLIED_PRICE applied_price, int shift) export
{
	double vl = base_period / iVI(symbol, timeframe, avg_period, stdv_period, applied_price, shift);
	vl = MathMax(base_period/2, MathMin(base_period*2, vl));
	return iRSI(symbol, timeframe, (int)vl, applied_price, shift);
}